#!/bin/sh
# check_mounted.sh - Nagios plugin
# Runs only on Linux
# check that file systems in /etc/fstab are mounted
# returns WARNING if not
# check writability of local r/w file systems via touch
# returns CRITICAL if not
#
# by Michael Tosch

PATH=/bin:/usr/bin:/usr/sbin:/sbin
set -f

case $1 in
(*-h*)
  echo "\
Usage: $0
Checks that file systems in /etc/fstab are mounted
Checks writability of local r/w file systems via touch"
  exit 3
;;
esac

for mtab in /etc/mtab /proc/mounts
do
  [ -r $mtab ] && break
done
if [ ! -r $mtab ]
then
  echo "no $mtab"
  exit 3
fi

fstab=/etc/fstab
if [ ! -f $fstab ]
then
  echo "no $fstab"
  exit 3
fi

not_mounted=`
awk '
  FILENAME!="-" { if ($1~/^[^#]/ && $1~/\// && $2~/^\// && $3!="swap" && ","$4","!~/,noauto,|,nofail,|,nobootwait,/) { if ($2!="/") sub(/\/$/,"",$2); s[$2] } next }
  ($2 in s) { delete s[$2] }
  END { for (i in s) print i }
' $fstab - < $mtab
`
if [ -n "$not_mounted" ]
then
  echo "WARNING not mounted:" $not_mounted
  exit 1
fi

# Minimum write test: set atime on the local rw mount points
local_rw_mounted=`
awk '
  FILENAME!="-" { if ($1~/^(\/dev|UUID=|LABEL=|PARTUUID=|PARTLABEL=)/ && $2~/^\// && $3!="swap" && ","$4","!~/,ro,/) { if ($2!="/") sub(/\/$/,"",$2); s[$2] } next }
  ($2 in s) { print $2; delete s[$2] }
' $fstab - < $mtab
`
res=""
IFS="
"
for fs in $local_rw_mounted
do
  touch -ac "$fs" 2>/dev/null || res="$res $fs"
done
if [ -n "$res" ]
then
  echo "CRITICAL not writable: $res"
  exit 2
else
  echo "OK: all FS in $fstab are mounted"
fi
